@@ -7,6 +7,7 @@ Device::Firmata::Platform - Platform specifics
 =cut
 
 use strict;
+use warnings;
 use Time::HiRes qw/time/;
 use Device::Firmata::Constants qw/ :all /;
 use Device::Firmata::Protocol;
@@ -30,6 +31,7 @@ use Device::Firmata::Base
   servo_resolutions           => {},
   stepper_resolutions         => {},
   encoder_resolutions         => {},
+  serial_resolutions          => {},
   ports                       => [],
   input_ports                 => [],
   pins                        => {},
@@ -44,6 +46,7 @@ use Device::Firmata::Base
   onewire_observer            => [],
   stepper_observer            => [],
   encoder_observer            => [],
+  serial_observer             => [],
   scheduler_observer          => undef,
   string_observer             => undef,
 
@@ -93,6 +96,7 @@ sub detach {
   $self->{onewire_observer}   = [];
   $self->{stepper_observer}   = [];
   $self->{encoder_observer}   = [];
+  $self->{serial_observer}    = [];
   $self->{scheduler_observer} = undef;
   $self->{tasks}              = [];
   $self->{metadata}           = {};
@@ -119,6 +123,7 @@ sub system_reset {
   $self->{onewire_observer}   = [];
   $self->{stepper_observer}   = [];
   $self->{encoder_observer}   = [];
+  $self->{serial_observer}    = [];
   $self->{scheduler_observer} = undef;
   $self->{tasks}              = [];
   $self->{metadata}           = {};
@@ -186,7 +191,7 @@ sub messages_handle {
 
       # Handle metadata information
       $command eq 'REPORT_VERSION' and do {
-        $self->{metadata}{firmware_version} = sprintf "V_%i_%02i",
+        $self->{metadata}{protocol_version} = sprintf "V_%i_%02i",
           @$data;
         last;
       };
@@ -251,7 +256,9 @@ sub sysex_handle {
       my @onewirepins;
       my @stepperpins;
       my @encoderpins;
-      
+      my @serialpins;
+      my @pulluppins;
+
       foreach my $pin (keys %$capabilities) {
         if (defined $capabilities->{$pin}) {
           if ($capabilities->{$pin}->{PIN_INPUT+0}) {
@@ -289,6 +296,13 @@ sub sysex_handle {
           	push @encoderpins, $pin;
             $self->{metadata}{encoder_resolutions}{$pin} = $capabilities->{$pin}->{PIN_ENCODER+0}->{resolution};
           }
+          if ($capabilities->{$pin}->{PIN_SERIAL+0}) {
+            push @serialpins, $pin;
+            $self->{metadata}{serial_resolutions}{$pin} = $capabilities->{$pin}->{PIN_SERIAL+0}->{resolution};
+          }
+          if ($capabilities->{$pin}->{PIN_PULLUP+0}) {
+            push @pulluppins, $pin;
+          }
         }
       }
       $self->{metadata}{input_pins}   = \@inputpins;
@@ -301,6 +315,8 @@ sub sysex_handle {
       $self->{metadata}{onewire_pins} = \@onewirepins;
       $self->{metadata}{stepper_pins} = \@stepperpins;
       $self->{metadata}{encoder_pins} = \@encoderpins;
+      $self->{metadata}{serial_pins}  = \@serialpins;
+      $self->{metadata}{pullup_pins}  = \@pulluppins;
       last;
     };
 
@@ -373,6 +389,15 @@ sub sysex_handle {
       };
       last;
     };
+
+    $sysex_message->{command_str} eq 'SERIAL_DATA' and do {
+      my $serialPort = $data->{port};
+      my $observer = $self->{serial_observer}[$serialPort];
+      if (defined $observer) {
+        $observer->{method}( $data, $observer->{context} );
+      }
+      last;
+    };
   }
 }
 
@@ -392,14 +417,16 @@ sub probe {
   my ($self) = @_;
   $self->{metadata}{firmware}         = '';
   $self->{metadata}{firmware_version} = '';
+  $self->{metadata}{protocol_version} = '';
 
   # Wait for 5 seconds only
   my $end_tics = time + 5;
   $self->firmware_version_query();
+  $self->protocol_version_query();
   while ( $end_tics >= time ) {
-    select( undef, undef, undef, 0.2 );    # wait for response
-    if ( $self->poll && $self->{metadata}{firmware} && $self->{metadata}{firmware_version} ) {
-      $self->{protocol}->{protocol_version} = $self->{metadata}{firmware_version};
+    select( undef, undef, undef, 0.2 );    # wait for responses
+    if ( $self->poll && $self->{metadata}{firmware} && $self->{metadata}{firmware_version} && $self->{metadata}{protocol_version} ) {
+      $self->{protocol}->{protocol_version} = $self->{protocol}->get_max_supported_protocol_version($self->{metadata}{protocol_version});
       if ( $self->{metadata}{capabilities} ) {
         if ( $self->{metadata}{analog_mappings} ) {
           return 1;
@@ -409,8 +436,10 @@ sub probe {
       } else {
         $self->capability_query();
       }
-    } else {
-      $self->firmware_version_query() unless $end_tics - 2 >= time;    # version query on last 2 sec only
+    } elsif ($end_tics - 2 < time) {
+      # version query on last 2 sec only
+      $self->firmware_version_query();
+      $self->protocol_version_query();
     }
   }
   return;
@@ -432,7 +461,7 @@ sub pin_mode {
 
   PIN_MODE_HANDLER: {
 
-    ( $mode == PIN_INPUT ) and do {
+    ( $mode == PIN_INPUT or $mode == PIN_PULLUP ) and do {
       my $port_number = $pin >> 3;
       $self->{io}->data_write($self->{protocol}->message_prepare( SET_PIN_MODE => 0, $pin, $mode ));
       $self->{io}->data_write($self->{protocol}->message_prepare( REPORT_DIGITAL => $port_number, 1 ));
@@ -456,13 +485,17 @@ sub pin_mode {
 Analogous to the digitalWrite function on the
 arduino
 
+Deprecation Warning:
+Writing to pin with mode "INPUT" is only supported for backward compatibility
+to switch pullup on and off. Use sub pin_mode with $mode=PIN_PULLUP instead.
+
 =cut
 
 sub digital_write {
 
   # --------------------------------------------------
   my ( $self, $pin, $state ) = @_;
-  die "pin '".$pin."' is not configured for mode 'INPUT' or 'OUTPUT'" unless ($self->is_configured_mode($pin,PIN_OUTPUT) or $self->is_configured_mode($pin,PIN_INPUT));
+  die "pin '".$pin."' is not configured for mode 'INPUT', 'PULLUP' or 'OUTPUT'" unless ($self->is_configured_mode($pin,PIN_OUTPUT) or $self->is_configured_mode($pin,PIN_INPUT) or $self->is_configured_mode($pin,PIN_PULLUP));
   my $port_number = $pin >> 3;
 
   my $pin_offset = $pin % 8;
@@ -491,7 +524,7 @@ sub digital_read {
 
   # --------------------------------------------------
   my ( $self, $pin ) = @_;
-  die "pin '".$pin."' is not configured for mode 'INPUT'" unless $self->is_configured_mode($pin,PIN_INPUT);
+  die "pin '".$pin."' is not configured for mode 'INPUT' or 'PULLUP'" unless ($self->is_configured_mode($pin,PIN_INPUT) or $self->is_configured_mode($pin,PIN_PULLUP));
   my $port_number = $pin >> 3;
   my $pin_offset  = $pin % 8;
   my $pin_mask    = 1 << $pin_offset;
@@ -540,6 +573,12 @@ pmw_write is an alias for analog_write
 
 *pwm_write = *analog_write;
 
+sub protocol_version_query {
+  my $self = shift;
+  my $protocol_version_query_packet = $self->{protocol}->packet_query_version;
+  return $self->{io}->data_write($protocol_version_query_packet);
+}
+
 sub firmware_version_query {
   my $self = shift;
   my $firmware_version_query_packet = $self->{protocol}->packet_query_firmware;
@@ -831,6 +870,29 @@ sub encoder_detach {
   return $self->{io}->data_write($self->{protocol}->packet_encoder_detach( $encoderNum ));
 }
 
+sub serial_write {
+  my ( $self, $port, @data ) = @_;
+  return $self->{io}->data_write($self->{protocol}->packet_serial_write( $port, @data ));
+}
+
+sub serial_read {
+  my ( $self, $port, $numbytes ) = @_;
+  if ($port >= 8) {
+    $self->{io}->data_write($self->{protocol}->packet_serial_listen( $port ));
+  }
+  return $self->{io}->data_write($self->{protocol}->packet_serial_read( $port, 0x00, $numbytes ));
+}
+
+sub serial_stopreading {
+  my ( $self, $port) = @_;
+  return $self->{io}->data_write($self->{protocol}->packet_serial_read( $port, 0x01, 0 ));
+}
+
+sub serial_config {
+  my ( $self, $port, $baud, $rxPin, $txPin ) = @_;
+  return $self->{io}->data_write($self->{protocol}->packet_serial_config( $port, $baud, $rxPin, $txPin ));
+}
+
 =head2 poll
 
 Call this function every once in a while to
@@ -921,6 +983,16 @@ sub observe_encoder {
   return 1;
 }
 
+sub observe_serial {
+  my ( $self, $port, $observer, $context ) = @_;
+  return undef if (defined $self->{metadata}->{serialpins} && @$self->{metadata}->{serialpins} == 0 );
+  $self->{serial_observer}[$port] =  {
+      method  => $observer,
+      context => $context,
+    };
+  return 1;
+}
+
 sub observe_scheduler {
   my ( $self, $observer, $context ) = @_;
   $self->{scheduler_observer} = {
@@ -944,7 +1016,7 @@ sub is_supported_mode {
   return undef if (defined $self->{metadata}->{capabilities} and (!(defined $self->{metadata}->{capabilities}->{$pin}) or !(defined $self->{metadata}->{capabilities}->{$pin}->{$mode})));
   return 1;
 }
- 
+
 sub is_configured_mode {
   my ($self,$pin,$mode) = @_;
   return undef if (!defined $self->{pin_modes}->{$pin} or $self->{pin_modes}->{$pin} != $mode);
